from typing import Optional, List

from intime_sdk.core.base import APIConnector
from intime_sdk.intime import constants


class Messages(APIConnector):
    """
    Class for manipulating SMS functionalities including creating & sending SMS, removing a scheduled sms etc.
    """
    def send_sms(self, sender_title: str, message_body: str, recipients: List, group_id: str,
                 is_flash_msg: bool = False, is_multi_sms: bool = False, send_date: Optional[str] = None,
                 status_url: Optional[str] = None, check_block_list: Optional[bool] = False,
                 encrypt_msg: Optional[bool] = False, trans_id: Optional[List] = None,
                 ):
        """
        Method for crating and sending sms.

        :param sender_title: (String) Sender of the message. Use maximum 11 alphanumeric characters or 15 numeric chars.
         Allowed alphanumeric chars is aA-zZ, 0-9, space, .(dot), -(binding char), or +(plus).
         Only English characters are allowed.
        :param message_body: (String) Message text.
         SMS is allowed to contain a maximum of 160 characters. Multi-SMS is allowed to contain a maximum of
         804 characters (6 SMS with 134 characters each).
        :param recipients: (List) Phone number of the recipients to receive the message. At least one recipient or group
         must be addressed.
        :param group_id: (String) The groups Identification number.
        :param is_flash_msg: (Boolean | Default: False) If this parameter is set to True, method will send a Flash
         message (maximum 160 characters) and otherwise it will send a Normal message.
        :param is_multi_sms: (Boolean | Default: False) If this parameter is set to True, method will send send up to
         six SMS or 804 characters and otherwise it will send a Normal message.
        :param send_date: (Optional; String | Default: None) Date/time for scheduled sending, If the message is to be
         sent directly/right now, ignore this parameter. (Format: 2012-01-13T14:41:00)
        :param status_url: (Optional; String | Default: None) URL for the server to send message status callbacks.
        :param check_block_list: (Boolean | Default: False) If this parameter is set to True, method will check
         the user's block list for group mailings.
        :param encrypt_msg: (Boolean | Default: False) Flag that indicates whether the message should be encrypted after
         it's sent. Sent messages cannot be read in Messenger.
        :param trans_id: (Optional; String | Default: None) Value to send to the status url with the callback, this
         could be a uniq id generated for each recipient and used in callback to link the status callback.
         This is mandatory if status_url is used!


        :return: API response in dictionary format with a status param
         Format::: {
                "status": True/False,
                "data": "Sample Response/3545354535"
            }
        """
        recipients_list = """"""
        if status_url:
            if trans_id:
                if len(trans_id) == len(recipients):
                    for inc in range(0, len(recipients)):
                        recipients_list += constants.RECIPIENT_XML.format(
                            recipient=recipients[inc],
                            trans_id=trans_id[inc]
                        )
                else:
                    raise ValueError("Each recipients should have a trans_id for Status CallBack")
            else:
                raise ValueError("trans_id is needed for Status CallBack")
        else:
            for recipient in recipients:
                recipients_list += constants.RECIPIENT_XML.format(
                    recipient=recipient,
                    trans_id=''
                )

        xml_data = constants.SEND_SMS_XML.format(
            is_flash_msg="1" if is_flash_msg else "0",
            is_multi_sms="1" if is_multi_sms else "0",
            send_date=constants.SEND_DATE_XML.format(send_date=send_date) if send_date else '',
            sender_title=sender_title,
            message_body=message_body,
            status_url=constants.STATUS_URL_XML.format(status_url=status_url) if status_url else '',
            check_block_list="1" if check_block_list else "0",
            encrypt_msg="1" if encrypt_msg else "0",
            recipients=recipients_list,
            group_id=group_id
        )

        return self._post(api_url=constants.SEND_SMS_URI,
                          data=xml_data)



# ==================== Send SMS ==================== #
SEND_SMS_XML = """<?xml version="1.0" encoding="iso-8859-1"?>
<message>
       <flash>{is_flash_msg}</flash>
       <multisms>{is_multi_sms}</multisms>
       {send_date}
       <sendertitle>{sender_title}</sendertitle>
       <body>{message_body}</body>
       {status_url}
       <checknix>{check_block_list}</checknix>
       <anonymize>{encrypt_msg}</anonymize>
       <recipients>
            {recipients}
       </recipients>
       <groups>
            <group groupid="{group_id}" />
      </groups>
</message>"""
SEND_SMS_URI = "message/create.ashx"
SEND_DATE_XML = """<senddate>{send_date}</senddate>"""
STATUS_URL_XML = """<statusurl>{status_url}</statusurl>"""
RECIPIENT_XML = """<recipient {trans_id}>{recipient}</recipient>"""



in_time_error = {
    "0:ERROR1": "Username and/or Secret-Key is missing in request",
    "0:ERROR2": "Bad Request",
    "0:ERROR3": "User is not verified",
    "1:ERROR1": "Data validation failure",
    "1:ERROR2": "Send date is past from current date and time",
    "1:ERROR3": "Recipients is incorrect",
    "2:ERROR0": "Loading the user's block list failed",
    "2:ERROR1": "Unable to send message",
    "2:ERROR2": "There is no recipient to send the message to"
}



import json

import requests

from typing import Optional, Dict

from intime_sdk.core import constants
from intime_sdk.core.error_map import in_time_error as _errors


class APIConnector:
    """
    Base class for API connector
    """
    def __init__(self, username: str, secret_key: str):
        if username:
            self.__username = username
        else:
            raise ValueError("Username is missing")

        if secret_key:
            self.__secret_key = secret_key
        else:
            raise ValueError("Secret hash key is missing")

    @staticmethod
    def __get_header() -> dict:
        """
        Method for manipulating API header

        :return: Content type of header
        """
        return {
            'Content-Type': 'text/xml'
        }

    def __get_required_query_params(self) -> dict:
        """
        Method for manipulating API parameter that is required

        :return: Authentication query dict
        """
        return {
            'user': self.__username,
            'hash': self.__secret_key
        }

    def _get(self, api_url: str, params: Optional[Dict] = None) -> dict:
        """
        Base/Main GET method for performing GET requests.

        :param api_url: URI Endpoint
        :param params: (Optional) API request parameters in dictionary format

        :return: API response in dictionary format
        """
        api_response = requests.get(
            url="{base}/{version}/{api_uri}".format(
                base=constants.INTIME_URL,
                version=constants.VERSION,
                api_uri=api_url
            ),
            headers=self.__get_header(),
            params=dict(self.__get_required_query_params().items(), **params)
        )

        if api_response.status_code == 200 and 'ERROR' not in api_response.text:
            return {
                "status": True,
                "data": api_response.json()
            }

        return {
                "status": False,
                "message": 'Error from InTime API. ' + _errors[str(api_response.text)]
                }

    def _post(self, api_url: str, data: Optional[str] = None) -> dict:
        """
        Base/Main POST method for performing POST requests.

        :param api_url: API endpoint
        :param data: (Optional) API request body containing XML in string Format

        :return:
        """
        header = self.__get_header()

        api_response = requests.post(
            url="{base}/{version}/{api_uri}".format(
                base=constants.INTIME_URL,
                version=constants.VERSION,
                api_uri=api_url
            ),
            headers=header,
            params=self.__get_required_query_params(),
            data=data
        )

        if api_response.status_code == 200 and 'ERROR' not in api_response.text:
            if 'Name cannot begin' not in api_response.text:
                return {
                    "status": True,
                    "data": json.dumps(api_response.text)
                }
            return {
                "status": False,
                "data": json.dumps(api_response.text)
            }

        return {
                "status": False,
                "message": 'Error from InTime API. ' + _errors[str(api_response.text)]
                }



import io
import os

from setuptools import find_packages, setup


# Package meta-data.
NAME = 'intime-sdk'
DESCRIPTION = 'Python SDK for InTime API services.'
URL = 'https://github.com/njnur/InTime-Python-SDK'
EMAIL = 'mnjnurrumen@gmail.com'
AUTHOR = 'MD Nurujjaman Nur'
REQUIRES_PYTHON = '>=3.6.0'
VERSION = '1.0.1'

# Packages required
REQUIRED = [
    'requests'
]

# Optional packages
EXTRAS = {
    # 'fancy-feature': ['django'],
}

package_root = os.path.abspath(os.path.dirname(__file__))

# README used as the long-description.
try:
    with io.open(os.path.join(package_root, 'README.md'), encoding='utf-8') as f:
        long_description = '\n' + f.read()
except FileNotFoundError:
    long_description = DESCRIPTION

# Load the package's __version__.py module as a dictionary.
about = {}
if not VERSION:
    project_slug = NAME.lower().replace("-", "_").replace(" ", "_")
    with open(os.path.join(package_root, project_slug, '__version__.py')) as f:
        exec(f.read(), about)
else:
    about['__version__'] = VERSION

setup(
    name=NAME,
    version=about['__version__'],
    description=DESCRIPTION,
    author=AUTHOR,
    author_email=EMAIL,
    python_requires=REQUIRES_PYTHON,
    url=URL,
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=find_packages(exclude=["docs", "source", "tests", "*.tests", "*.tests.*", "tests.*", ".pypirc"]),
    # For Single a module Package, use this instead of 'packages'
    # py_modules=['module_name'],
    # entry_points={},

    install_requires=REQUIRED,
    include_package_data=True,
    license='MIT',
    classifiers=[
        # Trove classifiers
        # Full list: https://pypi.python.org/pypi?%3Aaction=list_classifiers
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
    ]
)
